@@ -214,6 +214,42 @@ PCB_LAYER_ID FromProtoEnum( board::types::BoardLayer aValue )
     case board::types::BoardLayer::BL_User_8:      return User_8;
     case board::types::BoardLayer::BL_User_9:      return User_9;
     case board::types::BoardLayer::BL_Rescue:      return Rescue;
+    case board::types::BoardLayer::BL_User_10:     return User_10;
+    case board::types::BoardLayer::BL_User_11:     return User_11;
+    case board::types::BoardLayer::BL_User_12:     return User_12;
+    case board::types::BoardLayer::BL_User_13:     return User_13;
+    case board::types::BoardLayer::BL_User_14:     return User_14;
+    case board::types::BoardLayer::BL_User_15:     return User_15;
+    case board::types::BoardLayer::BL_User_16:     return User_16;
+    case board::types::BoardLayer::BL_User_17:     return User_17;
+    case board::types::BoardLayer::BL_User_18:     return User_18;
+    case board::types::BoardLayer::BL_User_19:     return User_19;
+    case board::types::BoardLayer::BL_User_20:     return User_20;
+    case board::types::BoardLayer::BL_User_21:     return User_21;
+    case board::types::BoardLayer::BL_User_22:     return User_22;
+    case board::types::BoardLayer::BL_User_23:     return User_23;
+    case board::types::BoardLayer::BL_User_24:     return User_24;
+    case board::types::BoardLayer::BL_User_25:     return User_25;
+    case board::types::BoardLayer::BL_User_26:     return User_26;
+    case board::types::BoardLayer::BL_User_27:     return User_27;
+    case board::types::BoardLayer::BL_User_28:     return User_28;
+    case board::types::BoardLayer::BL_User_29:     return User_29;
+    case board::types::BoardLayer::BL_User_30:     return User_30;
+    case board::types::BoardLayer::BL_User_31:     return User_31;
+    case board::types::BoardLayer::BL_User_32:     return User_32;
+    case board::types::BoardLayer::BL_User_33:     return User_33;
+    case board::types::BoardLayer::BL_User_34:     return User_34;
+    case board::types::BoardLayer::BL_User_35:     return User_35;
+    case board::types::BoardLayer::BL_User_36:     return User_36;
+    case board::types::BoardLayer::BL_User_37:     return User_37;
+    case board::types::BoardLayer::BL_User_38:     return User_38;
+    case board::types::BoardLayer::BL_User_39:     return User_39;
+    case board::types::BoardLayer::BL_User_40:     return User_40;
+    case board::types::BoardLayer::BL_User_41:     return User_41;
+    case board::types::BoardLayer::BL_User_42:     return User_42;
+    case board::types::BoardLayer::BL_User_43:     return User_43;
+    case board::types::BoardLayer::BL_User_44:     return User_44;
+    case board::types::BoardLayer::BL_User_45:     return User_45;
 
     case board::types::BoardLayer::BL_UNKNOWN:     return UNDEFINED_LAYER;
     default:
@@ -290,6 +326,42 @@ board::types::BoardLayer ToProtoEnum( PCB_LAYER_ID aValue )
     case User_8:            return board::types::BoardLayer::BL_User_8;
     case User_9:            return board::types::BoardLayer::BL_User_9;
     case Rescue:            return board::types::BoardLayer::BL_Rescue;
+    case User_10:           return board::types::BoardLayer::BL_User_10;
+    case User_11:           return board::types::BoardLayer::BL_User_11;
+    case User_12:           return board::types::BoardLayer::BL_User_12;
+    case User_13:           return board::types::BoardLayer::BL_User_13;
+    case User_14:           return board::types::BoardLayer::BL_User_14;
+    case User_15:           return board::types::BoardLayer::BL_User_15;
+    case User_16:           return board::types::BoardLayer::BL_User_16;
+    case User_17:           return board::types::BoardLayer::BL_User_17;
+    case User_18:           return board::types::BoardLayer::BL_User_18;
+    case User_19:           return board::types::BoardLayer::BL_User_19;
+    case User_20:           return board::types::BoardLayer::BL_User_20;
+    case User_21:           return board::types::BoardLayer::BL_User_21;
+    case User_22:           return board::types::BoardLayer::BL_User_22;
+    case User_23:           return board::types::BoardLayer::BL_User_23;
+    case User_24:           return board::types::BoardLayer::BL_User_24;
+    case User_25:           return board::types::BoardLayer::BL_User_25;
+    case User_26:           return board::types::BoardLayer::BL_User_26;
+    case User_27:           return board::types::BoardLayer::BL_User_27;
+    case User_28:           return board::types::BoardLayer::BL_User_28;
+    case User_29:           return board::types::BoardLayer::BL_User_29;
+    case User_30:           return board::types::BoardLayer::BL_User_30;
+    case User_31:           return board::types::BoardLayer::BL_User_31;
+    case User_32:           return board::types::BoardLayer::BL_User_32;
+    case User_33:           return board::types::BoardLayer::BL_User_33;
+    case User_34:           return board::types::BoardLayer::BL_User_34;
+    case User_35:           return board::types::BoardLayer::BL_User_35;
+    case User_36:           return board::types::BoardLayer::BL_User_36;
+    case User_37:           return board::types::BoardLayer::BL_User_37;
+    case User_38:           return board::types::BoardLayer::BL_User_38;
+    case User_39:           return board::types::BoardLayer::BL_User_39;
+    case User_40:           return board::types::BoardLayer::BL_User_40;
+    case User_41:           return board::types::BoardLayer::BL_User_41;
+    case User_42:           return board::types::BoardLayer::BL_User_42;
+    case User_43:           return board::types::BoardLayer::BL_User_43;
+    case User_44:           return board::types::BoardLayer::BL_User_44;
+    case User_45:           return board::types::BoardLayer::BL_User_45;
     default:
         wxCHECK_MSG( false, board::types::BoardLayer::BL_UNKNOWN,
                      "Unhandled case in ToProtoEnum<PCB_LAYER_ID>");
