@@ -73,18 +73,23 @@ PROJECT_LOCAL_SETTINGS::PROJECT_LOCAL_SETTINGS( PROJECT* aProject, const wxStrin
                         ret.push_back( VisibilityLayerToString( *vl ) );
                 }
 
+                // Explicit marker to tell apart a wiped-out array from the user hiding everything
+                if( ret.empty() )
+                    ret.push_back( "none" );
+
                 return ret;
             },
             [&]( const nlohmann::json& aVal )
             {
                 if( !aVal.is_array() || aVal.empty() )
                 {
-                    m_VisibleItems = GAL_SET::DefaultVisible();
+                    m_VisibleItems |= UserVisbilityLayers();
                     return;
                 }
 
                 m_VisibleItems &= ~UserVisbilityLayers();
                 GAL_SET visible;
+                bool none = false;
 
                 for( const nlohmann::json& entry : aVal )
                 {
@@ -94,14 +99,21 @@ PROJECT_LOCAL_SETTINGS::PROJECT_LOCAL_SETTINGS( PROJECT* aProject, const wxStrin
 
                         if( std::optional<GAL_LAYER_ID> l = RenderLayerFromVisbilityString( vs ) )
                             visible.set( *l );
+                        else if( vs == "none" )
+                            none = true;
                     }
                     catch( ... )
                     {
-                        // Non-integer or out of range entry in the array; ignore
+                        // Unknown entry (possibly the settings file was re-saved by an old version
+                        // of kicad that used numeric entries, or is a future format)
                     }
                 }
 
-                m_VisibleItems |= UserVisbilityLayers() & visible;
+                // Restore corrupted state
+                if( !visible.any() && !none )
+                    m_VisibleItems |= UserVisbilityLayers();
+                else
+                    m_VisibleItems |= UserVisbilityLayers() & visible;
             },
             {} ) );
 
@@ -421,14 +433,14 @@ PROJECT_LOCAL_SETTINGS::PROJECT_LOCAL_SETTINGS( PROJECT* aProject, const wxStrin
     registerMigration( 3, 4,
             [&]()
             {
-                // Schema version 3 to 4: LAYER_SHAPES added to visibility controls
+                // Schema version 3 to 4: LAYER_FILLED_SHAPES added to visibility controls
 
                 std::string ptr( "board.visible_items" );
 
                 if( Contains( ptr ) )
                 {
-                    if( At( ptr ).is_array() )
-                        At( ptr ).push_back( LAYER_SHAPES - GAL_LAYER_ID_START );
+                    if( At( ptr ).is_array() && !At( ptr ).empty() )
+                        At( ptr ).push_back( LAYER_FILLED_SHAPES - GAL_LAYER_ID_START );
                     else
                         At( "board" ).erase( "visible_items" );
 
