@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014-2016 CERN
- * Copyright (C) 2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -25,49 +25,57 @@
 
 #include <base_screen.h>
 #include <base_units.h>
+#include <gal/painter.h>
 #include <bitmaps.h>
 #include <class_draw_panel_gal.h>
 #include <dialogs/dialog_configure_paths.h>
+#include <dialogs/dialog_unit_entry.h>
 #include <eda_draw_frame.h>
 #include <gal/graphics_abstraction_layer.h>
 #include <id.h>
+#include <math/vector2wx.h>
+#include <core/kicad_algo.h>
 #include <kiface_base.h>
-#include <project.h>
 #include <settings/app_settings.h>
 #include <tool/actions.h>
 #include <tool/common_tools.h>
 #include <tool/tool_manager.h>
+#include <tool/selection_tool.h>
+#include <tool/grid_helper.h>
 #include <view/view.h>
 #include <view/view_controls.h>
-#include <zoom_defines.h>
+#include "macros.h"
 
 
 COMMON_TOOLS::COMMON_TOOLS() :
     TOOL_INTERACTIVE( "common.Control" ),
     m_frame( nullptr ),
-    m_imperialUnit( EDA_UNITS::INCHES ),
-    m_metricUnit( EDA_UNITS::MILLIMETRES )
+    m_imperialUnit( EDA_UNITS::INCH ),
+    m_metricUnit( EDA_UNITS::MM )
 {
 }
 
+
 void COMMON_TOOLS::Reset( RESET_REASON aReason )
 {
     m_frame = getEditFrame<EDA_DRAW_FRAME>();
-
-    GRID_SETTINGS& settings = m_toolMgr->GetSettings()->m_Window.grid;
-
     m_grids.clear();
 
-    for( const wxString& gridDef : settings.sizes )
+    if( aReason == RESET_REASON::SHUTDOWN )
+        return;
+
+    GRID_SETTINGS& settings = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid;
+    EDA_IU_SCALE   scale = m_frame->GetIuScale();
+
+    for( GRID& gridDef : settings.grids )
     {
-        int gridSize = (int) ValueFromString( EDA_UNITS::MILLIMETRES, gridDef );
-        m_grids.emplace_back( gridSize, gridSize );
+        double gridSizeX = EDA_UNIT_UTILS::UI::DoubleValueFromString( scale, EDA_UNITS::MM, gridDef.x );
+        double gridSizeY = EDA_UNIT_UTILS::UI::DoubleValueFromString( scale, EDA_UNITS::MM, gridDef.y );
+
+        m_grids.emplace_back( KiROUND<double, int>( gridSizeX ), KiROUND<double, int>( gridSizeY ) );
     }
 
-    m_grids.emplace_back( ValueFromString( EDA_UNITS::MILLIMETRES, settings.user_grid_x ),
-                          ValueFromString( EDA_UNITS::MILLIMETRES, settings.user_grid_y ) );
-
-    OnGridChanged();
+    OnGridChanged( false );
 }
 
 
@@ -78,7 +86,7 @@ void COMMON_TOOLS::SetLastUnits( EDA_UNITS aUnit )
     else if( EDA_UNIT_UTILS::IsMetricUnit( aUnit ) )
         m_metricUnit = aUnit;
     else
-        wxASSERT_MSG( false, "Invalid unit" );
+        wxASSERT_MSG( false, wxS( "Invalid unit" ) );
 }
 
 
@@ -92,34 +100,46 @@ int COMMON_TOOLS::SelectionTool( const TOOL_EVENT& aEvent )
 }
 
 
-// Cursor control
 int COMMON_TOOLS::CursorControl( const TOOL_EVENT& aEvent )
 {
-    long type = aEvent.Parameter<intptr_t>();
-    bool fastMove = type & ACTIONS::CURSOR_FAST_MOVE;
-    type &= ~ACTIONS::CURSOR_FAST_MOVE;
-    bool mirroredX = getView()->IsMirroredX();
+    ACTIONS::CURSOR_EVENT_TYPE   type = aEvent.Parameter<ACTIONS::CURSOR_EVENT_TYPE>();
+    std::unique_ptr<GRID_HELPER> grid = m_frame->MakeGridHelper();
+    VECTOR2D                     gridSize;
 
+    if( grid )
+        gridSize = grid->GetGridSize( grid->GetSelectionGrid( m_frame->GetCurrentSelection() ) );
+    else
+        gridSize = getView()->GetGAL()->GetGridSize();
+
+    bool     mirroredX = getView()->IsMirroredX();
     VECTOR2D cursor = getViewControls()->GetRawCursorPosition( false );
-    VECTOR2D gridSize = getView()->GetGAL()->GetGridSize();
-
-    if( fastMove )
-        gridSize = gridSize * 10;
 
     switch( type )
     {
+    case ACTIONS::CURSOR_UP_FAST:
+        gridSize *= 10;
+        KI_FALLTHROUGH;
     case ACTIONS::CURSOR_UP:
         cursor -= VECTOR2D( 0, gridSize.y );
         break;
 
+    case ACTIONS::CURSOR_DOWN_FAST:
+        gridSize *= 10;
+        KI_FALLTHROUGH;
     case ACTIONS::CURSOR_DOWN:
         cursor += VECTOR2D( 0, gridSize.y );
         break;
 
+    case ACTIONS::CURSOR_LEFT_FAST:
+        gridSize *= 10;
+        KI_FALLTHROUGH;
     case ACTIONS::CURSOR_LEFT:
         cursor -= VECTOR2D( mirroredX ? -gridSize.x : gridSize.x, 0 );
         break;
 
+    case ACTIONS::CURSOR_RIGHT_FAST:
+        gridSize *= 10;
+        KI_FALLTHROUGH;
     case ACTIONS::CURSOR_RIGHT:
         cursor += VECTOR2D( mirroredX ? -gridSize.x : gridSize.x, 0 );
         break;
@@ -128,9 +148,9 @@ int COMMON_TOOLS::CursorControl( const TOOL_EVENT& aEvent )
     case ACTIONS::CURSOR_DBL_CLICK:
     case ACTIONS::CURSOR_RIGHT_CLICK:
     {
-        TOOL_ACTIONS action = TA_MOUSE_CLICK;
+        TOOL_ACTIONS       action = TA_MOUSE_CLICK;
         TOOL_MOUSE_BUTTONS button = BUT_LEFT;
-        int modifiers = 0;
+        int                modifiers = 0;
 
         modifiers |= wxGetKeyState( WXK_SHIFT ) ? MD_SHIFT : 0;
         modifiers |= wxGetKeyState( WXK_CONTROL ) ? MD_CTRL : 0;
@@ -138,23 +158,22 @@ int COMMON_TOOLS::CursorControl( const TOOL_EVENT& aEvent )
 
         if( type == ACTIONS::CURSOR_DBL_CLICK )
             action = TA_MOUSE_DBLCLICK;
-
-        if( type == ACTIONS::CURSOR_RIGHT_CLICK )
+        else if( type == ACTIONS::CURSOR_RIGHT_CLICK )
             button = BUT_RIGHT;
 
-        TOOL_EVENT evt( TC_MOUSE, action, button | modifiers );
-        evt.SetParameter( static_cast<intptr_t>( type ) );
-        evt.SetMousePosition( getViewControls()->GetCursorPosition() );
+        TOOL_EVENT evt( TC_MOUSE, action, static_cast<int>( button | modifiers ) );
+        evt.SetParameter( type );
+        evt.SetMousePosition( getViewControls()->GetMousePosition() );
         m_toolMgr->ProcessEvent( evt );
 
         return 0;
     }
     default:
-        wxFAIL_MSG( "CursorControl(): unexpected request" );
+        wxFAIL_MSG( wxS( "CursorControl(): unexpected request" ) );
     }
 
     getViewControls()->SetCursorPosition( cursor, true, true, type );
-    m_toolMgr->RunAction( ACTIONS::refreshPreview );
+    m_toolMgr->PostAction( ACTIONS::refreshPreview );
 
     return 0;
 }
@@ -162,7 +181,7 @@ int COMMON_TOOLS::CursorControl( const TOOL_EVENT& aEvent )
 
 int COMMON_TOOLS::PanControl( const TOOL_EVENT& aEvent )
 {
-    long type = aEvent.Parameter<intptr_t>();
+    ACTIONS::CURSOR_EVENT_TYPE type = aEvent.Parameter<ACTIONS::CURSOR_EVENT_TYPE>();
     KIGFX::VIEW* view = getView();
     VECTOR2D center = view->GetCenter();
     VECTOR2D gridSize = getView()->GetGAL()->GetGridSize() * 10;
@@ -229,7 +248,7 @@ int COMMON_TOOLS::doZoomInOut( bool aDirection, bool aCenterOnCursor )
         zoom /= 1.3;
 
     // Now look for the next closest menu step
-    std::vector<double>& zoomList = m_toolMgr->GetSettings()->m_Window.zoom_factors;
+    std::vector<double>& zoomList = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->zoom_factors;
     int idx;
 
     if( aDirection )
@@ -282,6 +301,12 @@ int COMMON_TOOLS::ZoomFitObjects( const TOOL_EVENT& aEvent )
 }
 
 
+int COMMON_TOOLS::ZoomFitSelection( const TOOL_EVENT& aEvent )
+{
+    return doZoomFit( ZOOM_FIT_SELECTION );
+}
+
+
 int COMMON_TOOLS::doZoomFit( ZOOM_FIT_TYPE_T aFitType )
 {
     KIGFX::VIEW*        view   = getView();
@@ -294,19 +319,35 @@ int COMMON_TOOLS::doZoomFit( ZOOM_FIT_TYPE_T aFitType )
     view->SetScale( 1.0 );  // The best scale will be determined later, but this initial
                             // value ensures all view parameters are up to date (especially
                             // at init time)
-    VECTOR2D screenSize = view->ToWorld( canvas->GetClientSize(), false );
+    VECTOR2D screenSize = view->ToWorld( ToVECTOR2I( canvas->GetClientSize() ), false );
 
     // Currently "Zoom to Objects" is only supported in Eeschema & Pcbnew.  Support for other
     // programs in the suite can be added as needed.
 
+    if( aFitType == ZOOM_FIT_ALL )
+    {
+        if( frame->IsType( FRAME_PCB_EDITOR ) )
+            bBox = m_frame->GetDocumentExtents( false );
+    }
+
     if( aFitType == ZOOM_FIT_OBJECTS )
     {
-        if( frame->IsType( FRAME_SCH ) || frame->IsType( FRAME_PCB_EDITOR ) )
+        if( frame->IsType( FRAME_SCH ) )
             bBox = m_frame->GetDocumentExtents( false );
         else
             aFitType = ZOOM_FIT_ALL; // Just do a "Zoom to Fit" for unsupported editors
     }
 
+    if( aFitType == ZOOM_FIT_SELECTION )
+    {
+        SELECTION& selection = m_frame->GetCurrentSelection();
+
+        if( selection.Empty() )
+            return 0;
+
+        bBox = selection.GetBoundingBox();
+    }
+
     // If the screen is empty then use the default view bbox
 
     if( bBox.GetWidth() == 0 || bBox.GetHeight() == 0 )
@@ -321,6 +362,7 @@ int COMMON_TOOLS::doZoomFit( ZOOM_FIT_TYPE_T aFitType )
     if( !std::isfinite( scale ) )
     {
         view->SetCenter( VECTOR2D( 0, 0 ) );
+        canvas->Refresh();
         return 0;
     }
 
@@ -335,8 +377,8 @@ int COMMON_TOOLS::doZoomFit( ZOOM_FIT_TYPE_T aFitType )
     {
         // Leave a bigger margin for library editors & viewers
 
-        if( frame->IsType( FRAME_FOOTPRINT_VIEWER ) || frame->IsType( FRAME_FOOTPRINT_VIEWER_MODAL )
-                || frame->IsType( FRAME_SCH_VIEWER ) || frame->IsType( FRAME_SCH_VIEWER_MODAL ) )
+        if( frame->IsType( FRAME_FOOTPRINT_VIEWER )
+                || frame->IsType( FRAME_SCH_VIEWER ) )
         {
             margin_scale_factor = 1.30;
         }
@@ -349,25 +391,55 @@ int COMMON_TOOLS::doZoomFit( ZOOM_FIT_TYPE_T aFitType )
 
     view->SetScale( scale / margin_scale_factor );
     view->SetCenter( bBox.Centre() );
+    canvas->Refresh();
 
     return 0;
 }
 
 
+int COMMON_TOOLS::CenterSelection( const TOOL_EVENT& aEvent )
+{
+    return doCenter( CENTER_TYPE::CENTER_SELECTION );
+}
+
+
 int COMMON_TOOLS::CenterContents( const TOOL_EVENT& aEvent )
 {
-    EDA_DRAW_PANEL_GAL* canvas = m_frame->GetCanvas();
-    BOX2I bBox = getModel<EDA_ITEM>()->ViewBBox();
+    return doCenter( CENTER_TYPE::CENTER_CONTENTS );
+}
 
-    if( bBox.GetWidth() == 0 || bBox.GetHeight() == 0 )
-        bBox = canvas->GetDefaultViewBBox();
+
+int COMMON_TOOLS::doCenter( CENTER_TYPE aCenterType )
+{
+    EDA_DRAW_PANEL_GAL* canvas = m_frame->GetCanvas();
+
+    BOX2I bBox;
+
+    if( aCenterType == CENTER_TYPE::CENTER_SELECTION )
+    {
+        SELECTION& selection = m_frame->GetCurrentSelection();
+
+        // No selection: do nothing
+        if( selection.Empty() )
+            return 0;
+
+        bBox = selection.GetBoundingBox().Centre();
+    }
+    else
+    {
+        bBox = getModel<EDA_ITEM>()->ViewBBox();
+
+        if( bBox.GetWidth() == 0 || bBox.GetHeight() == 0 )
+            bBox = canvas->GetDefaultViewBBox();
+    }
 
     getView()->SetCenter( bBox.Centre() );
 
     // Take scrollbars into account
-    VECTOR2D scrollbarSize = VECTOR2D( canvas->GetSize() - canvas->GetClientSize() );
+    VECTOR2D scrollbarSize = VECTOR2D( ToVECTOR2D( canvas->GetSize() - canvas->GetClientSize() ) );
     VECTOR2D worldScrollbarSize = getView()->ToWorld( scrollbarSize, false );
     getView()->SetCenter( getView()->GetCenter() + worldScrollbarSize / 2.0 );
+    canvas->Refresh();
 
     return 0;
 }
@@ -375,15 +447,15 @@ int COMMON_TOOLS::CenterContents( const TOOL_EVENT& aEvent )
 
 int COMMON_TOOLS::ZoomPreset( const TOOL_EVENT& aEvent )
 {
-    unsigned int idx = aEvent.Parameter<intptr_t>();
-    return doZoomToPreset( (int) idx, false );
+    int idx = aEvent.Parameter<int>();
+    return doZoomToPreset( idx, false );
 }
 
 
 // Note: idx == 0 is Auto; idx == 1 is first entry in zoomList
 int COMMON_TOOLS::doZoomToPreset( int idx, bool aCenterOnCursor )
 {
-    std::vector<double>& zoomList = m_toolMgr->GetSettings()->m_Window.zoom_factors;
+    std::vector<double>& zoomList = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->zoom_factors;
 
     if( idx == 0 )      // Zoom Auto
     {
@@ -409,52 +481,57 @@ int COMMON_TOOLS::doZoomToPreset( int idx, bool aCenterOnCursor )
         getView()->SetScale( scale );
     }
 
+    m_frame->GetCanvas()->Refresh();
+
     return 0;
 }
 
 
-// Grid control
 int COMMON_TOOLS::GridNext( const TOOL_EVENT& aEvent )
 {
-    int& currentGrid = m_toolMgr->GetSettings()->m_Window.grid.last_size_idx;
+    int& currentGrid = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.last_size_idx;
 
-    if( currentGrid + 1 < int( m_grids.size() ) )
-        currentGrid++;
+    currentGrid++;
 
-    return OnGridChanged();
+    if( currentGrid >= int( m_grids.size() ) )
+        currentGrid = 0;
+
+    return OnGridChanged( true );
 }
 
 
 int COMMON_TOOLS::GridPrev( const TOOL_EVENT& aEvent )
 {
-    int& currentGrid = m_toolMgr->GetSettings()->m_Window.grid.last_size_idx;
+    int& currentGrid = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.last_size_idx;
 
-    if( currentGrid > 0 )
-        currentGrid--;
+    currentGrid--;
 
-    return OnGridChanged();
+    if( currentGrid < 0 )
+        currentGrid = (int) m_grids.size() - 1;
+
+    return OnGridChanged( true );
 }
 
 
 int COMMON_TOOLS::GridPreset( const TOOL_EVENT& aEvent )
 {
-    return GridPreset( aEvent.Parameter<intptr_t>() );
+    return GridPreset( aEvent.Parameter<int>(), false );
 }
 
 
-int COMMON_TOOLS::GridPreset( int idx )
+int COMMON_TOOLS::GridPreset( int idx, bool aFromHotkey )
 {
-    int& currentGrid = m_toolMgr->GetSettings()->m_Window.grid.last_size_idx;
+    int& currentGrid = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.last_size_idx;
 
-    currentGrid = std::max( 0, std::min( idx, (int) m_grids.size() - 1 ) );
+    currentGrid = std::clamp( idx, 0, (int) m_grids.size() - 1 );
 
-    return OnGridChanged();
+    return OnGridChanged( aFromHotkey );
 }
 
 
-int COMMON_TOOLS::OnGridChanged()
+int COMMON_TOOLS::OnGridChanged( bool aFromHotkey )
 {
-    int& currentGrid = m_toolMgr->GetSettings()->m_Window.grid.last_size_idx;
+    int& currentGrid = m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.last_size_idx;
 
     currentGrid = std::max( 0, std::min( currentGrid, static_cast<int>( m_grids.size() ) - 1 ) );
 
@@ -464,43 +541,99 @@ int COMMON_TOOLS::OnGridChanged()
 
     // Update GAL canvas from screen
     getView()->GetGAL()->SetGridSize( m_grids[ currentGrid ] );
-    getView()->GetGAL()->SetGridVisibility( m_toolMgr->GetSettings()->m_Window.grid.show );
+    getView()->GetGAL()->SetGridVisibility( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.show );
     getView()->MarkTargetDirty( KIGFX::TARGET_NONCACHED );
 
     // Put cursor on new grid
     VECTOR2D gridCursor = getViewControls()->GetCursorPosition( true );
     getViewControls()->SetCrossHairCursorPosition( gridCursor, false );
 
+    // Show feedback
+    if( aFromHotkey )
+        m_toolMgr->PostEvent( EVENTS::GridChangedByKeyEvent );
+
     return 0;
 }
 
 
 int COMMON_TOOLS::GridFast1( const TOOL_EVENT& aEvent )
 {
-    return GridPreset( m_frame->config()->m_Window.grid.fast_grid_1 );
+    return GridPreset( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.fast_grid_1, true );
 }
 
 
 int COMMON_TOOLS::GridFast2( const TOOL_EVENT& aEvent )
 {
-    return GridPreset( m_frame->config()->m_Window.grid.fast_grid_2 );
+    return GridPreset( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.fast_grid_2, true );
+}
+
+
+int COMMON_TOOLS::GridFastCycle( const TOOL_EVENT& aEvent )
+{
+    if( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.last_size_idx
+        == m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.fast_grid_1 )
+    {
+        return GridPreset( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.fast_grid_2, true );
+    }
+
+    return GridPreset( m_frame->GetWindowSettings( m_toolMgr->GetSettings() )->grid.fast_grid_1, true );
 }
 
 
 int COMMON_TOOLS::ToggleGrid( const TOOL_EVENT& aEvent )
 {
     m_frame->SetGridVisibility( !m_frame->IsGridVisible() );
+    return 0;
+}
 
+
+int COMMON_TOOLS::ToggleGridOverrides( const TOOL_EVENT& aEvent )
+{
+    m_frame->SetGridOverrides( !m_frame->IsGridOverridden() );
     return 0;
 }
 
 
 int COMMON_TOOLS::GridProperties( const TOOL_EVENT& aEvent )
 {
-    wxCommandEvent cmd( wxEVT_COMMAND_MENU_SELECTED );
+    auto showGridPrefs =
+            [this]( const wxString& aParentName )
+            {
+                m_frame->CallAfter(
+                        [this, aParentName]()
+                        {
+                            m_frame->ShowPreferences( _( "Grids" ), aParentName );
+                        } );
+            };
 
-    cmd.SetId( ID_GRID_SETTINGS );
-    m_frame->ProcessEvent( cmd );
+    switch( m_frame->GetFrameType() )
+    {
+    case FRAME_SCH:               showGridPrefs( _( "Schematic Editor" ) );     break;
+    case FRAME_SCH_SYMBOL_EDITOR: showGridPrefs( _( "Symbol Editor" ) );        break;
+    case FRAME_PCB_EDITOR:        showGridPrefs( _( "PCB Editor" ) );           break;
+    case FRAME_FOOTPRINT_EDITOR:  showGridPrefs( _( "Footprint Editor" ) );     break;
+    case FRAME_PL_EDITOR:         showGridPrefs( _( "Drawing Sheet Editor" ) ); break;
+    case FRAME_GERBER:            showGridPrefs( _( "Gerber Viewer" ) );        break;
+    default:                      wxFAIL_MSG( "Unknown frame: " + GetName() );  break;
+    }
+
+    return 0;
+}
+
+
+int COMMON_TOOLS::GridOrigin( const TOOL_EVENT& aEvent )
+{
+    VECTOR2I           origin = m_frame->GetGridOrigin();
+    WX_PT_ENTRY_DIALOG dlg( m_frame, _( "Grid Origin" ), _( "X:" ), _( "Y:" ), origin, true );
+
+    if( dlg.ShowModal() == wxID_OK )
+    {
+        m_frame->SetGridOrigin( dlg.GetValue() );
+
+        m_toolMgr->ResetTools( TOOL_BASE::REDRAW );
+        m_toolMgr->RunAction( ACTIONS::gridSetOrigin, new VECTOR2D( m_frame->GetGridOrigin() ) );
+        m_frame->GetCanvas()->ForceRefresh();
+    }
 
     return 0;
 }
@@ -515,7 +648,7 @@ int COMMON_TOOLS::SwitchUnits( const TOOL_EVENT& aEvent )
     else if( EDA_UNIT_UTILS::IsImperialUnit( newUnit ) )
         m_imperialUnit = newUnit;
     else
-        wxASSERT_MSG( false, "Invalid unit for the frame" );
+        wxASSERT_MSG( false, wxS( "Invalid unit for the frame" ) );
 
     m_frame->ChangeUserUnits( newUnit );
     return 0;
@@ -543,7 +676,10 @@ int COMMON_TOOLS::TogglePolarCoords( const TOOL_EVENT& aEvent )
 
 int COMMON_TOOLS::ResetLocalCoords( const TOOL_EVENT& aEvent )
 {
-    auto vcSettings = m_toolMgr->GetCurrentToolVC();
+    if( !m_frame->GetScreen() )     // Can happen in footprint chooser frame
+        return 0;
+
+    const KIGFX::VC_SETTINGS& vcSettings = m_toolMgr->GetCurrentToolVC();
 
     // Use either the active tool forced cursor position or the general settings
     if( vcSettings.m_forceCursorPosition )
@@ -562,6 +698,7 @@ int COMMON_TOOLS::ToggleCursor( const TOOL_EVENT& aEvent )
     auto& galOpts = m_frame->GetGalDisplayOptions();
 
     galOpts.m_forceDisplayCursor = !galOpts.m_forceDisplayCursor;
+    galOpts.WriteConfig( *m_frame->GetWindowSettings( m_toolMgr->GetSettings() ) );
     galOpts.NotifyChanged();
 
     return 0;
@@ -570,71 +707,96 @@ int COMMON_TOOLS::ToggleCursor( const TOOL_EVENT& aEvent )
 
 int COMMON_TOOLS::ToggleCursorStyle( const TOOL_EVENT& aEvent )
 {
-    KIGFX::GAL_DISPLAY_OPTIONS& galOpts = m_frame->GetGalDisplayOptions();
+    GAL_DISPLAY_OPTIONS_IMPL& galOpts = m_frame->GetGalDisplayOptions();
 
     galOpts.m_fullscreenCursor = !galOpts.m_fullscreenCursor;
+    galOpts.WriteConfig( *m_frame->GetWindowSettings( m_toolMgr->GetSettings() ) );
     galOpts.NotifyChanged();
 
     return 0;
 }
 
 
+int COMMON_TOOLS::ToggleBoundingBoxes( const TOOL_EVENT& aEvent )
+{
+    EDA_DRAW_PANEL_GAL* canvas = m_frame->GetCanvas();
+
+    if( canvas )
+    {
+        KIGFX::RENDER_SETTINGS* rs = canvas->GetView()->GetPainter()->GetSettings();
+
+        rs->SetDrawBoundingBoxes( !rs->GetDrawBoundingBoxes() );
+
+        canvas->GetView()->UpdateAllItems( KIGFX::ALL );
+        canvas->ForceRefresh();
+    }
+
+    return 0;
+}
+
+
 void COMMON_TOOLS::setTransitions()
 {
-    Go( &COMMON_TOOLS::SelectionTool,      ACTIONS::selectionTool.MakeEvent() );
+    Go( &COMMON_TOOLS::SelectionTool,       ACTIONS::selectionTool.MakeEvent() );
 
     // Cursor control
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorUp.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorDown.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorLeft.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorRight.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorUpFast.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorDownFast.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorLeftFast.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorRightFast.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorUp.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorDown.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorLeft.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorRight.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorUpFast.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorDownFast.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorLeftFast.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorRightFast.MakeEvent() );
 
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorClick.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::cursorDblClick.MakeEvent() );
-    Go( &COMMON_TOOLS::CursorControl,      ACTIONS::showContextMenu.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorClick.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::cursorDblClick.MakeEvent() );
+    Go( &COMMON_TOOLS::CursorControl,       ACTIONS::showContextMenu.MakeEvent() );
 
     // Pan control
-    Go( &COMMON_TOOLS::PanControl,         ACTIONS::panUp.MakeEvent() );
-    Go( &COMMON_TOOLS::PanControl,         ACTIONS::panDown.MakeEvent() );
-    Go( &COMMON_TOOLS::PanControl,         ACTIONS::panLeft.MakeEvent() );
-    Go( &COMMON_TOOLS::PanControl,         ACTIONS::panRight.MakeEvent() );
+    Go( &COMMON_TOOLS::PanControl,          ACTIONS::panUp.MakeEvent() );
+    Go( &COMMON_TOOLS::PanControl,          ACTIONS::panDown.MakeEvent() );
+    Go( &COMMON_TOOLS::PanControl,          ACTIONS::panLeft.MakeEvent() );
+    Go( &COMMON_TOOLS::PanControl,          ACTIONS::panRight.MakeEvent() );
 
     // Zoom control
-    Go( &COMMON_TOOLS::ZoomRedraw,         ACTIONS::zoomRedraw.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomInOut,          ACTIONS::zoomIn.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomInOut,          ACTIONS::zoomOut.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomInOutCenter,    ACTIONS::zoomInCenter.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomInOutCenter,    ACTIONS::zoomOutCenter.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomCenter,         ACTIONS::zoomCenter.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomFitScreen,      ACTIONS::zoomFitScreen.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomFitObjects,     ACTIONS::zoomFitObjects.MakeEvent() );
-    Go( &COMMON_TOOLS::ZoomPreset,         ACTIONS::zoomPreset.MakeEvent() );
-    Go( &COMMON_TOOLS::CenterContents,     ACTIONS::centerContents.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomRedraw,          ACTIONS::zoomRedraw.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomInOut,           ACTIONS::zoomIn.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomInOut,           ACTIONS::zoomOut.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomInOutCenter,     ACTIONS::zoomInCenter.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomInOutCenter,     ACTIONS::zoomOutCenter.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomCenter,          ACTIONS::zoomCenter.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomFitScreen,       ACTIONS::zoomFitScreen.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomFitObjects,      ACTIONS::zoomFitObjects.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomFitSelection,    ACTIONS::zoomFitSelection.MakeEvent() );
+    Go( &COMMON_TOOLS::ZoomPreset,          ACTIONS::zoomPreset.MakeEvent() );
+    Go( &COMMON_TOOLS::CenterContents,      ACTIONS::centerContents.MakeEvent() );
+    Go( &COMMON_TOOLS::CenterSelection,     ACTIONS::centerSelection.MakeEvent() );
 
     // Grid control
-    Go( &COMMON_TOOLS::GridNext,           ACTIONS::gridNext.MakeEvent() );
-    Go( &COMMON_TOOLS::GridPrev,           ACTIONS::gridPrev.MakeEvent() );
-    Go( &COMMON_TOOLS::GridPreset,         ACTIONS::gridPreset.MakeEvent() );
-    Go( &COMMON_TOOLS::GridFast1,          ACTIONS::gridFast1.MakeEvent() );
-    Go( &COMMON_TOOLS::GridFast2,          ACTIONS::gridFast2.MakeEvent() );
-    Go( &COMMON_TOOLS::ToggleGrid,         ACTIONS::toggleGrid.MakeEvent() );
-    Go( &COMMON_TOOLS::GridProperties,     ACTIONS::gridProperties.MakeEvent() );
+    Go( &COMMON_TOOLS::GridNext,            ACTIONS::gridNext.MakeEvent() );
+    Go( &COMMON_TOOLS::GridPrev,            ACTIONS::gridPrev.MakeEvent() );
+    Go( &COMMON_TOOLS::GridPreset,          ACTIONS::gridPreset.MakeEvent() );
+    Go( &COMMON_TOOLS::GridFast1,           ACTIONS::gridFast1.MakeEvent() );
+    Go( &COMMON_TOOLS::GridFast2,           ACTIONS::gridFast2.MakeEvent() );
+    Go( &COMMON_TOOLS::GridFastCycle,       ACTIONS::gridFastCycle.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleGrid,          ACTIONS::toggleGrid.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleGridOverrides, ACTIONS::toggleGridOverrides.MakeEvent() );
+    Go( &COMMON_TOOLS::GridProperties,      ACTIONS::gridProperties.MakeEvent() );
+    Go( &COMMON_TOOLS::GridOrigin,          ACTIONS::gridOrigin.MakeEvent() );
 
     // Units and coordinates
-    Go( &COMMON_TOOLS::SwitchUnits,        ACTIONS::inchesUnits.MakeEvent() );
-    Go( &COMMON_TOOLS::SwitchUnits,        ACTIONS::milsUnits.MakeEvent() );
-    Go( &COMMON_TOOLS::SwitchUnits,        ACTIONS::millimetersUnits.MakeEvent() );
-    Go( &COMMON_TOOLS::ToggleUnits,        ACTIONS::toggleUnits.MakeEvent() );
-    Go( &COMMON_TOOLS::TogglePolarCoords,  ACTIONS::togglePolarCoords.MakeEvent() );
-    Go( &COMMON_TOOLS::ResetLocalCoords,   ACTIONS::resetLocalCoords.MakeEvent() );
+    Go( &COMMON_TOOLS::SwitchUnits,         ACTIONS::inchesUnits.MakeEvent() );
+    Go( &COMMON_TOOLS::SwitchUnits,         ACTIONS::milsUnits.MakeEvent() );
+    Go( &COMMON_TOOLS::SwitchUnits,         ACTIONS::millimetersUnits.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleUnits,         ACTIONS::toggleUnits.MakeEvent() );
+    Go( &COMMON_TOOLS::TogglePolarCoords,   ACTIONS::togglePolarCoords.MakeEvent() );
+    Go( &COMMON_TOOLS::ResetLocalCoords,    ACTIONS::resetLocalCoords.MakeEvent() );
 
     // Misc
-    Go( &COMMON_TOOLS::ToggleCursor,       ACTIONS::toggleCursor.MakeEvent() );
-    Go( &COMMON_TOOLS::ToggleCursorStyle,  ACTIONS::toggleCursorStyle.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleCursor,        ACTIONS::toggleCursor.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleCursorStyle,   ACTIONS::toggleCursorStyle.MakeEvent() );
+    Go( &COMMON_TOOLS::ToggleBoundingBoxes, ACTIONS::toggleBoundingBoxes.MakeEvent() );
 }
 
 
