@@ -78,12 +78,6 @@ LIB_TREE::LIB_TREE( wxWindow* aParent, const wxString& aRecentSearchesKey, LIB_T
 
         m_query_ctrl->ShowCancelButton( true );
 
-#ifdef __WXGTK__
-        // wxSearchCtrl vertical height is not calculated correctly on some GTK setups
-        // See https://gitlab.com/kicad/code/kicad/-/issues/9019
-        m_query_ctrl->SetMinSize( wxSize( -1, GetTextExtent( wxT( "qb" ) ).y + 10 ) );
-#endif
-
         m_debounceTimer = new wxTimer( this );
 
         search_sizer->Add( m_query_ctrl, 1, wxEXPAND | wxRIGHT, 5 );
@@ -284,7 +278,23 @@ LIB_TREE::~LIB_TREE()
         m_details_ctrl->Unbind( wxEVT_HTML_LINK_CLICKED, &LIB_TREE::onDetailsLink, this );
 
     m_hoverTimer.Stop();
-    destroyPreview();
+}
+
+
+void LIB_TREE::ShutdownPreviews()
+{
+    m_hoverTimer.Stop();
+    m_previewDisabled = true;
+
+    if( m_previewWindow )
+    {
+        // Shutdown the preview window's canvas
+        m_adapter->ShutdownPreview( m_previewWindow );
+
+        m_previewWindow->Hide();
+        m_previewWindow->Destroy();
+        m_previewWindow = nullptr;
+    }
 }
 
 
@@ -305,6 +315,12 @@ LIB_ID LIB_TREE::GetSelectedLibId( int* aUnit ) const
     if( !sel )
         return LIB_ID();
 
+    if( m_adapter->GetTreeNodeFor( sel )->m_IsAlreadyPlacedGroup
+            || m_adapter->GetTreeNodeFor( sel )->m_IsRecentlyUsedGroup )
+    {
+        return LIB_ID();
+    }
+
     if( aUnit )
         *aUnit = m_adapter->GetUnitFor( sel );
 
@@ -339,6 +355,19 @@ LIB_TREE_NODE* LIB_TREE::GetCurrentTreeNode() const
     return m_adapter->GetTreeNodeFor( sel );
 }
 
+int LIB_TREE::GetSelectedTreeNodes( std::vector<LIB_TREE_NODE*>& aSelection ) const
+{
+    wxDataViewItemArray selection;
+    int                 count = m_tree_ctrl->GetSelections( selection );
+
+    for( const wxDataViewItem& item : selection )
+    {
+        aSelection.push_back( m_adapter->GetTreeNodeFor( item ) );
+    }
+
+    return count;
+}
+
 
 void LIB_TREE::SelectLibId( const LIB_ID& aLibId )
 {
@@ -738,18 +767,6 @@ void LIB_TREE::hidePreview()
 }
 
 
-void LIB_TREE::destroyPreview()
-{
-    hidePreview();
-
-    if( m_previewWindow )
-    {
-        m_previewWindow->Destroy();
-        m_previewWindow = nullptr;
-    }
-}
-
-
 void LIB_TREE::onIdle( wxIdleEvent& aEvent )
 {
     // The wxDataViewCtrl won't give us its mouseMoved events so we're forced to use idle
