@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2019-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -34,7 +34,7 @@
 #include <schematic.h>
 #include <sch_commit.h>
 #include <tool/tool_manager.h>
-#include <tools/ee_selection_tool.h>
+#include <tools/sch_selection_tool.h>
 #include <tools/sch_edit_tool.h>
 #include <widgets/unit_binder.h>
 #include <widgets/font_choice.h>
@@ -66,12 +66,10 @@ static wxString   g_netFilter;
 static bool       g_filterSelected;
 
 
-#define DEFAULT_STYLE _( "Default" )
-
 class DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS : public DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS_BASE
 {
     SCH_EDIT_FRAME*        m_parent;
-    EE_SELECTION           m_selection;
+    SCH_SELECTION          m_selection;
 
     UNIT_BINDER            m_textSize;
     UNIT_BINDER            m_lineWidth;
@@ -115,7 +113,6 @@ DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS( SCH_
 {
     m_parent = parent;
 
-    m_lineStyle->Append( DEFAULT_STYLE );
     m_lineStyle->Append( INDETERMINATE_ACTION );
 
     m_textColorSwatch->SetSwatchColor( COLOR4D::UNSPECIFIED, false );
@@ -165,7 +162,7 @@ DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::~DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS()
 
 bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataToWindow()
 {
-    EE_SELECTION_TOOL* selectionTool = m_parent->GetToolManager()->GetTool<EE_SELECTION_TOOL>();
+    SCH_SELECTION_TOOL* selectionTool = m_parent->GetToolManager()->GetTool<SCH_SELECTION_TOOL>();
     m_selection = selectionTool->GetSelection();
 
     m_references->SetValue( g_modifyReferences );
@@ -287,9 +284,6 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( SCH_COMMIT* aCommit,
             eda_text->SetVertJustify( vAlign );
         }
 
-        if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
-            eda_text->SetVisible( m_visible->GetValue() );
-
         if( m_italic->Get3StateValue() != wxCHK_UNDETERMINED )
             eda_text->SetItalic( m_italic->GetValue() );
 
@@ -323,6 +317,9 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( SCH_COMMIT* aCommit,
 
     if( SCH_FIELD* sch_field = dynamic_cast<SCH_FIELD*>( aItem ) )
     {
+        if( m_visible->Get3StateValue() != wxCHK_UNDETERMINED )
+            sch_field->SetVisible( m_visible->GetValue() );
+
         if( m_showFieldNames->Get3StateValue() != wxCHK_UNDETERMINED )
             sch_field->SetNameShown( m_showFieldNames->GetValue() );
     }
@@ -335,12 +332,7 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::processItem( SCH_COMMIT* aCommit,
             stroke.SetWidth( m_lineWidth.GetValue() );
 
         if( m_lineStyle->GetStringSelection() != INDETERMINATE_ACTION )
-        {
-            if( m_lineStyle->GetStringSelection() == DEFAULT_STYLE )
-                stroke.SetLineStyle( LINE_STYLE::DEFAULT );
-            else
-                stroke.SetLineStyle( (LINE_STYLE) m_lineStyle->GetSelection() );
-        }
+            stroke.SetLineStyle( (LINE_STYLE) m_lineStyle->GetSelection() );
 
         if( m_setColor->GetValue() )
             stroke.SetColor( m_colorSwatch->GetSwatchColor() );
@@ -419,6 +411,9 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( SCH_COMMIT* aCommit,
         }
     }
 
+    static const std::vector<KICAD_T> wireLabelTypes = { SCH_LABEL_LOCATE_WIRE_T };
+    static const std::vector<KICAD_T> busLabelTypes = { SCH_LABEL_LOCATE_BUS_T };
+
     switch( aItem->Type() )
     {
     case SCH_SYMBOL_T:
@@ -508,10 +503,10 @@ void DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::visitItem( SCH_COMMIT* aCommit,
     case SCH_GLOBAL_LABEL_T:
     case SCH_HIER_LABEL_T:
     case SCH_DIRECTIVE_LABEL_T:
-        if( m_wires->GetValue() && aItem->IsType( { SCH_LABEL_LOCATE_WIRE_T } ) )
+        if( m_wires->GetValue() && aItem->IsType( wireLabelTypes ) )
             processItem( aCommit, aSheetPath, aItem );
 
-        if( m_buses->GetValue() && aItem->IsType( { SCH_LABEL_LOCATE_BUS_T } ) )
+        if( m_buses->GetValue() && aItem->IsType( busLabelTypes ) )
             processItem( aCommit, aSheetPath, aItem );
 
         if( m_globalLabels->GetValue() && aItem->Type() == SCH_GLOBAL_LABEL_T )
@@ -580,7 +575,7 @@ bool DIALOG_GLOBAL_EDIT_TEXT_AND_GRAPHICS::TransferDataFromWindow()
     SCH_COMMIT     commit( m_parent );
 
     // Go through sheets
-    for( const SCH_SHEET_PATH& sheetPath : m_parent->Schematic().GetSheets() )
+    for( const SCH_SHEET_PATH& sheetPath : m_parent->Schematic().Hierarchy() )
     {
         SCH_SCREEN* screen = sheetPath.LastScreen();
 
