@@ -27,6 +27,7 @@
 #include <widgets/font_choice.h>
 #include <widgets/std_bitmap_button.h>
 #include <widgets/color_swatch.h>
+#include <widgets/filter_combobox.h>
 #include <settings/color_settings.h>
 #include <sch_edit_frame.h>
 #include <tool/tool_manager.h>
@@ -42,7 +43,8 @@
 
 
 DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
-                                                  SCH_LABEL_BASE* aLabel ) :
+                                                  SCH_LABEL_BASE* aLabel,
+                                                  bool aNew ) :
         DIALOG_LABEL_PROPERTIES_BASE( aParent ),
         m_Parent( aParent ),
         m_currentLabel( aLabel ),
@@ -54,6 +56,12 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
 {
     COLOR_SETTINGS* colorSettings = m_Parent->GetColorSettings();
     COLOR4D         schematicBackground = colorSettings->GetColor( LAYER_SCHEMATIC_BACKGROUND );
+    bool            multiLine = false;
+
+    if( EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() ) )
+        multiLine = cfg->m_Appearance.edit_label_multiple;
+
+    m_cbMultiLine->SetValue( multiLine );
 
     m_fields = new FIELDS_GRID_TABLE( this, aParent, m_grid, m_currentLabel );
     m_width = 100;  // Will be later set to a better value
@@ -63,12 +71,24 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     if( m_currentLabel->Type() == SCH_GLOBAL_LABEL_T || m_currentLabel->Type() == SCH_LABEL_T )
     {
         m_activeTextEntry = m_valueCombo;
-        SetInitialFocus( m_valueCombo );
+        SetInitialFocus( m_valueCombo->GetTextCtrl() );
 
         m_labelSingleLine->Show( false );
         m_valueSingleLine->Show( false );
 
-        m_valueCombo->SetValidator( m_netNameValidator );
+        if( multiLine && aNew )
+        {
+            m_activeTextEntry = m_valueMultiLine;
+            SetInitialFocus( m_valueMultiLine );
+            m_labelCombo->Show( false );
+            m_valueCombo->Show( false );
+        }
+        else
+        {
+            m_labelMultiLine->Show( false );
+            m_valueMultiLine->Show( false );
+            m_valueCombo->SetValidator( m_netNameValidator );
+        }
     }
     else if( m_currentLabel->Type() == SCH_HIER_LABEL_T )
     {
@@ -78,7 +98,19 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
         m_labelCombo->Show( false );
         m_valueCombo->Show( false );
 
-        m_valueSingleLine->SetValidator( m_netNameValidator );
+        if( multiLine && aNew )
+        {
+            m_activeTextEntry = m_valueMultiLine;
+            SetInitialFocus( m_valueMultiLine );
+            m_labelSingleLine->Show( false );
+            m_valueSingleLine->Show( false );
+        }
+        else
+        {
+            m_labelMultiLine->Show( false );
+            m_valueMultiLine->Show( false );
+            m_valueSingleLine->SetValidator( m_netNameValidator );
+        }
     }
     else if( m_currentLabel->Type() == SCH_DIRECTIVE_LABEL_T )
     {
@@ -91,10 +123,16 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
         m_valueCombo->Show( false );
         m_syntaxHelp->Show( false );
         m_textEntrySizer->Show( false );
+        m_labelCombo->Show( false );
+        m_valueCombo->Show( false );
+        m_cbMultiLine->Show( false );
 
         m_textSizeLabel->SetLabel( _( "Pin length:" ) );
     }
 
+    if( !aNew )
+        m_cbMultiLine->Show( false );
+
     switch( m_currentLabel->Type() )
     {
     case SCH_GLOBAL_LABEL_T:    SetTitle( _( "Global Label Properties" ) );           break;
@@ -109,7 +147,7 @@ DIALOG_LABEL_PROPERTIES::DIALOG_LABEL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     m_grid->SetDefaultRowSize( m_grid->GetDefaultRowSize() + 4 );
 
     m_grid->SetTable( m_fields );
-    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this, nullptr,
+    m_grid->PushEventHandler( new FIELDS_GRID_TRICKS( m_grid, this, {},
                                                       [&]( wxCommandEvent& aEvent )
                                                       {
                                                           OnAddField( aEvent );
@@ -233,6 +271,7 @@ DIALOG_LABEL_PROPERTIES::~DIALOG_LABEL_PROPERTIES()
         cfg->m_Appearance.edit_label_visible_columns = m_grid->GetShownColumnsAsString();
         cfg->m_Appearance.edit_label_width = GetSize().x;
         cfg->m_Appearance.edit_label_height = GetSize().y;
+        cfg->m_Appearance.edit_label_multiple = m_cbMultiLine->IsChecked();
     }
 
     // Prevents crash bug in wxGrid's d'tor
@@ -251,15 +290,15 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
     if( !wxDialog::TransferDataToWindow() )
         return false;
 
+    wxString text;
+
     if( m_activeTextEntry )
     {
         // show control characters in a human-readable format
-        wxString text = UnescapeString( m_currentLabel->GetText() );
+        text = UnescapeString( m_currentLabel->GetText() );
 
         // show text variable cross-references in a human-readable format
         text = m_currentLabel->Schematic()->ConvertKIIDsToRefs( text );
-
-        m_activeTextEntry->SetValue( text );
     }
 
     if( m_currentLabel->Type() == SCH_GLOBAL_LABEL_T || m_currentLabel->Type() == SCH_LABEL_T )
@@ -300,12 +339,15 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataToWindow()
         for( const std::shared_ptr<BUS_ALIAS>& busAlias : busAliases )
             existingLabels.insert( wxT( "{" ) + busAlias->GetName() + wxT( "}" ) );
 
-        wxArrayString existingLabelArray;
-
         for( const wxString& label : existingLabels )
-            existingLabelArray.push_back( label );
+            m_existingLabelArray.push_back( label );
 
-        m_valueCombo->Append( existingLabelArray );
+        m_valueCombo->SetStringList( m_existingLabelArray );
+        m_valueCombo->SetSelectedString( text );
+    }
+    else if( m_activeTextEntry )
+    {
+        m_activeTextEntry->SetValue( text );
     }
 
     // Push a copy of each field into m_updateFields
@@ -372,6 +414,26 @@ void DIALOG_LABEL_PROPERTIES::OnEnterKey( wxCommandEvent& aEvent )
 }
 
 
+void DIALOG_LABEL_PROPERTIES::OnCBValueCharHook( wxKeyEvent& aEvent )
+{
+    // If the key is WXK_RETURN because m_valueCombo is the source event, we do not skip
+    // the key event because the default action is to show the m_valueCombo dropdown list,
+    // and we only want to accept the entered string.
+    if( aEvent.GetKeyCode() == WXK_RETURN )
+        wxPostEvent( this, wxCommandEvent( wxEVT_COMMAND_BUTTON_CLICKED, wxID_OK ) );
+    else if( aEvent.GetKeyCode() == WXK_SPACE )
+    {
+        // our FILTER_COMBOBOX uses the space as special command, not wanted here,
+        // to open the dropdown list of labels.
+        // So just add the space char to the current text, at current insertion point,
+        // and do not skip this event
+        m_activeTextEntry->WriteText( wxT(" " ) );
+    }
+    else
+        aEvent.Skip();
+}
+
+
 void DIALOG_LABEL_PROPERTIES::OnValueCharHook( wxKeyEvent& aEvent )
 {
     if( aEvent.GetKeyCode() == WXK_TAB )
@@ -595,7 +657,68 @@ bool DIALOG_LABEL_PROPERTIES::TransferDataFromWindow()
         m_currentLabel->AutoplaceFields( m_Parent->GetScreen(), fieldsAutoplaced );
 
     if( !commit.Empty() )
+    {
         commit.Push( _( "Edit Label Properties" ) );
+    }
+    else if( m_activeTextEntry && m_labelList )
+    {
+        text = m_activeTextEntry->GetValue();
+        // On macOS CTRL+Enter produces '\r' instead of '\n' regardless of EOL setting
+        text.Replace( wxS( "\r" ), wxS( "\n" ) );
+        wxArrayString lines = wxSplit( text, '\n' );
+
+        for( const wxString& line : lines )
+        {
+            text = EscapeString( line, CTX_NETNAME );
+            text.Trim( false ).Trim( true );
+
+            if( text.empty() )
+                continue;
+
+            // convert any text variable cross-references to their UUIDs
+            text = m_currentLabel->Schematic()->ConvertRefsToKIIDs( text );
+
+            switch ( m_currentLabel->Type() )
+            {
+            case SCH_GLOBAL_LABEL_T:
+            {
+                SCH_GLOBALLABEL* label = new SCH_GLOBALLABEL( *static_cast<SCH_GLOBALLABEL*>( m_currentLabel ) );
+                label->SetText( text );
+                m_labelList->push_back( std::unique_ptr<SCH_LABEL_BASE>( label ) );
+                break;
+            }
+            case SCH_HIER_LABEL_T:
+            {
+                SCH_HIERLABEL* label = new SCH_HIERLABEL( *static_cast<SCH_HIERLABEL*>( m_currentLabel ) );
+                label->SetText( text );
+                m_labelList->push_back( std::unique_ptr<SCH_LABEL_BASE>( label ) );
+                break;
+            }
+            case SCH_LABEL_T:
+            {
+                SCH_LABEL* label = new SCH_LABEL( *static_cast<SCH_LABEL*>( m_currentLabel ) );
+                label->SetText( text );
+                m_labelList->push_back( std::unique_ptr<SCH_LABEL_BASE>( label ) );
+                break;
+            }
+            case SCH_DIRECTIVE_LABEL_T:
+            {
+                SCH_DIRECTIVE_LABEL* label = new SCH_DIRECTIVE_LABEL( *static_cast<SCH_DIRECTIVE_LABEL*>( m_currentLabel ) );
+                label->SetText( text );
+                m_labelList->push_back( std::unique_ptr<SCH_LABEL_BASE>( label ) );
+                break;
+            }
+            default:
+                break;
+            }
+        }
+    }
+    else if( m_labelList && m_currentLabel->Type() == SCH_DIRECTIVE_LABEL_T )
+    {
+        SCH_DIRECTIVE_LABEL* label =
+                new SCH_DIRECTIVE_LABEL( *static_cast<SCH_DIRECTIVE_LABEL*>( m_currentLabel ) );
+        m_labelList->push_back( std::unique_ptr<SCH_LABEL_BASE>( label ) );
+    }
 
     return true;
 }
@@ -694,7 +817,7 @@ void DIALOG_LABEL_PROPERTIES::OnDeleteField( wxCommandEvent& event )
     for( int row : selectedRows )
     {
         //avoids an assert if we deselect early here
-        m_grid->DeselectRow( row );
+        m_grid->ClearSelection();
         m_fields->erase( m_fields->begin() + row );
 
         // notify the grid
@@ -813,3 +936,50 @@ void DIALOG_LABEL_PROPERTIES::OnSizeGrid( wxSizeEvent& event )
 }
 
 
+void DIALOG_LABEL_PROPERTIES::onMultiLabelCheck( wxCommandEvent& event )
+{
+    if( m_currentLabel->Type() == SCH_GLOBAL_LABEL_T || m_currentLabel->Type() == SCH_LABEL_T )
+    {
+        m_labelCombo->Show( !m_cbMultiLine->IsChecked() );
+        m_valueCombo->Show( !m_cbMultiLine->IsChecked() );
+        m_labelMultiLine->Show( m_cbMultiLine->IsChecked() );
+        m_valueMultiLine->Show( m_cbMultiLine->IsChecked() );
+
+        if( m_cbMultiLine->IsChecked() )
+        {
+            m_valueMultiLine->SetValue( m_valueCombo->GetValue() );
+            m_activeTextEntry = m_valueMultiLine;
+            SetInitialFocus( m_valueMultiLine );
+        }
+        else
+        {
+            wxString multiText = m_valueMultiLine->GetValue();
+            m_valueCombo->SetValue( multiText.BeforeFirst( '\n' ) );
+            m_activeTextEntry = m_valueCombo;
+            SetInitialFocus( m_valueCombo );
+        }
+    }
+    else if( m_currentLabel->Type() == SCH_HIER_LABEL_T )
+    {
+        m_labelSingleLine->Show( !m_cbMultiLine->IsChecked() );
+        m_valueSingleLine->Show( !m_cbMultiLine->IsChecked() );
+        m_labelMultiLine->Show( m_cbMultiLine->IsChecked() );
+        m_valueMultiLine->Show( m_cbMultiLine->IsChecked() );
+
+        if( m_cbMultiLine->IsChecked() )
+        {
+            m_valueMultiLine->SetValue( m_valueSingleLine->GetValue() );
+            m_activeTextEntry = m_valueMultiLine;
+            SetInitialFocus( m_valueMultiLine );
+        }
+        else
+        {
+            wxString multiText = m_valueMultiLine->GetValue();
+            m_valueSingleLine->SetValue( multiText.BeforeFirst( '\n' ) );
+            m_activeTextEntry = m_valueSingleLine;
+            SetInitialFocus( m_valueSingleLine );
+        }
+    }
+
+    Layout();
+}
