@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016-2023 CERN
- * Copyright (C) 2016-2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  * @author Maciej Suminski <maciej.suminski@cern.ch>
  *
@@ -46,7 +46,7 @@
 #include <tool/action_toolbar.h>
 #include <tool/common_control.h>
 #include <tools/simulator_control.h>
-#include <tools/ee_actions.h>
+#include <tools/sch_actions.h>
 #include <string_utils.h>
 #include <pgm_base.h>
 #include "ngspice.h"
@@ -143,23 +143,23 @@ SIMULATOR_FRAME::SIMULATOR_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     mainSizer->Add( m_ui, 1, wxEXPAND, 5 );
 
     m_simulator = SIMULATOR::CreateInstance( "ngspice" );
-    wxASSERT( m_simulator );
+
+    if( !m_simulator )
+        throw SIMULATOR_INIT_ERR( "Failed to create simulator instance" );
 
     LoadSettings( config() );
 
-    NGSPICE_SETTINGS* settings = dynamic_cast<NGSPICE_SETTINGS*>( m_simulator->Settings().get() );
+    std::shared_ptr<NGSPICE_SETTINGS> cfg = Prj().GetProjectFile().m_SchematicSettings->m_NgspiceSettings;
 
-    wxCHECK2( settings, /* do nothing in release builds*/ );
-
-    if( settings && settings->GetWorkbookFilename().IsEmpty() )
-        settings->SetCompatibilityMode( NGSPICE_COMPATIBILITY_MODE::LT_PSPICE );
+    if( cfg->GetWorkbookFilename().IsEmpty() )
+        cfg->SetCompatibilityMode( NGSPICE_COMPATIBILITY_MODE::LT_PSPICE );
 
     m_simulator->Init();
 
     m_reporter = new SIM_THREAD_REPORTER( this );
     m_simulator->SetReporter( m_reporter );
 
-    m_circuitModel = std::make_shared<SPICE_CIRCUIT_MODEL>( &m_schematicFrame->Schematic(), this );
+    m_circuitModel = std::make_shared<SPICE_CIRCUIT_MODEL>( &m_schematicFrame->Schematic() );
 
     setupTools();
     setupUIConditions();
@@ -167,8 +167,7 @@ SIMULATOR_FRAME::SIMULATOR_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
     ReCreateHToolbar();
     ReCreateMenuBar();
 
-    Bind( wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( SIMULATOR_FRAME::onExit ), this,
-          wxID_EXIT );
+    Bind( wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( SIMULATOR_FRAME::onExit ), this, wxID_EXIT );
 
     Bind( EVT_SIM_UPDATE, &SIMULATOR_FRAME::onUpdateSim, this );
     Bind( EVT_SIM_REPORT, &SIMULATOR_FRAME::onSimReport, this );
@@ -235,44 +234,39 @@ void SIMULATOR_FRAME::ShowChangedLanguage()
 
 void SIMULATOR_FRAME::LoadSettings( APP_SETTINGS_BASE* aCfg )
 {
-    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
-    wxASSERT( cfg );
-
-    if( cfg )
+    if( EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg ) )
     {
         EDA_BASE_FRAME::LoadSettings( cfg );
         m_ui->LoadSettings( cfg );
     }
 
-    PROJECT_FILE& project = Prj().GetProjectFile();
-
-    NGSPICE* currentSim = dynamic_cast<NGSPICE*>( m_simulator.get() );
-
-    if( currentSim )
-        m_simulator->Settings() = project.m_SchematicSettings->m_NgspiceSettings;
+    if( m_simulator )
+        m_simulator->Settings() = Prj().GetProjectFile().m_SchematicSettings->m_NgspiceSettings;
 }
 
 
 void SIMULATOR_FRAME::SaveSettings( APP_SETTINGS_BASE* aCfg )
 {
-    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
-    wxASSERT( cfg );
-
-    if( cfg )
+    if( EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg ) )
     {
         EDA_BASE_FRAME::SaveSettings( cfg );
         m_ui->SaveSettings( cfg );
     }
 
-    PROJECT_FILE& project = Prj().GetProjectFile();
+    bool modified = Prj().GetProjectFile().m_SchematicSettings->m_NgspiceSettings->SaveToFile();
 
-    if( project.m_SchematicSettings )
-    {
-        bool modified = project.m_SchematicSettings->m_NgspiceSettings->SaveToFile();
+    if( m_schematicFrame && modified )
+        m_schematicFrame->OnModify();
+}
 
-        if( m_schematicFrame && modified )
-            m_schematicFrame->OnModify();
-    }
+
+void SIMULATOR_FRAME::CommonSettingsChanged( int aFlags )
+{
+    KIWAY_PLAYER::CommonSettingsChanged( aFlags );
+
+    auto* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( m_toolManager->GetSettings() );
+    wxASSERT( cfg != nullptr );
+    m_ui->ApplyPreferences( cfg->m_Simulator.preferences );
 }
 
 
@@ -281,7 +275,8 @@ WINDOW_SETTINGS* SIMULATOR_FRAME::GetWindowSettings( APP_SETTINGS_BASE* aCfg )
     EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( aCfg );
     wxASSERT( cfg );
 
-    return cfg ? &cfg->m_Simulator.window : nullptr;
+    wxFAIL_MSG( wxT( "SIMULATOR not running with EESCHEMA_SETTINGS" ) );
+    return &aCfg->m_Window;     // non-null fail-safe
 }
 
 
@@ -311,10 +306,11 @@ int SIMULATOR_FRAME::GetCurrentOptions() const
 
 void SIMULATOR_FRAME::UpdateTitle()
 {
-    bool       unsaved = true;
-    bool       readOnly = false;
-    wxString   title;
-    wxFileName filename = Prj().AbsolutePath( m_simulator->Settings()->GetWorkbookFilename() );
+    bool                              unsaved = true;
+    bool                              readOnly = false;
+    wxString                          title;
+    std::shared_ptr<NGSPICE_SETTINGS> cfg = Prj().GetProjectFile().m_SchematicSettings->m_NgspiceSettings;
+    wxFileName                        filename = Prj().AbsolutePath( cfg->GetWorkbookFilename() );
 
     if( filename.IsOk() && filename.FileExists() )
     {
@@ -333,7 +329,7 @@ void SIMULATOR_FRAME::UpdateTitle()
     if( unsaved )
         title += wxS( " " ) + _( "[Unsaved]" );
 
-    title += wxT( " \u2014 " ) + _( "Spice Simulator" );
+    title += wxT( " \u2014 " ) + _( "SPICE Simulator" );
 
     SetTitle( title );
 }
@@ -543,7 +539,7 @@ bool SIMULATOR_FRAME::LoadWorkbook( const wxString& aPath )
 
         // Successfully loading a workbook does not count as modifying it.  Clear the modified
         // flag after all the EVT_WORKBOOK_MODIFIED events have been processed.
-        CallAfter( [=]()
+        CallAfter( [this]()
                    {
                        m_workbookModified = false;
                    } );
@@ -646,7 +642,7 @@ void SIMULATOR_FRAME::doCloseWindow()
 
     SaveSettings( config() );
 
-    m_simulator->Settings() = nullptr;
+    m_simulator->Settings().reset();
 
     Destroy();
 }
@@ -726,29 +722,31 @@ void SIMULATOR_FRAME::setupUIConditions()
 
 #define ENABLE( x ) ACTION_CONDITIONS().Enable( x )
 #define CHECK( x )  ACTION_CONDITIONS().Check( x )
+    // clang-format off
+    mgr->SetConditions( SCH_ACTIONS::openWorkbook,          ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( SCH_ACTIONS::saveWorkbook,          ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( SCH_ACTIONS::saveWorkbookAs,        ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
 
-    mgr->SetConditions( EE_ACTIONS::openWorkbook,          ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( EE_ACTIONS::saveWorkbook,          ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( EE_ACTIONS::saveWorkbookAs,        ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( SCH_ACTIONS::exportPlotAsPNG,       ENABLE( havePlot ) );
+    mgr->SetConditions( SCH_ACTIONS::exportPlotAsCSV,       ENABLE( havePlot ) );
+    mgr->SetConditions( SCH_ACTIONS::exportPlotToClipboard, ENABLE( havePlot ) );
+    mgr->SetConditions( SCH_ACTIONS::exportPlotToSchematic, ENABLE( havePlot ) );
 
-    mgr->SetConditions( EE_ACTIONS::exportPlotAsPNG,       ENABLE( havePlot ) );
-    mgr->SetConditions( EE_ACTIONS::exportPlotAsCSV,       ENABLE( havePlot ) );
-
-    mgr->SetConditions( ACTIONS::zoomUndo,                 ENABLE( haveZoomUndo ) );
-    mgr->SetConditions( ACTIONS::zoomRedo,                 ENABLE( haveZoomRedo ) );
-    mgr->SetConditions( EE_ACTIONS::toggleGrid,            CHECK( showGridCondition ) );
-    mgr->SetConditions( EE_ACTIONS::toggleLegend,          CHECK( showLegendCondition ) );
-    mgr->SetConditions( EE_ACTIONS::toggleDottedSecondary, CHECK( showDottedCondition ) );
-    mgr->SetConditions( EE_ACTIONS::toggleDarkModePlots,   CHECK( darkModePlotCondition ) );
-
-    mgr->SetConditions( EE_ACTIONS::newAnalysisTab,        ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
-    mgr->SetConditions( EE_ACTIONS::simAnalysisProperties, ENABLE( haveSim ) );
-    mgr->SetConditions( EE_ACTIONS::runSimulation,         ENABLE( !simRunning ) );
-    mgr->SetConditions( EE_ACTIONS::stopSimulation,        ENABLE( simRunning ) );
-    mgr->SetConditions( EE_ACTIONS::simProbe,              ENABLE( simFinished ) );
-    mgr->SetConditions( EE_ACTIONS::simTune,               ENABLE( simFinished ) );
-    mgr->SetConditions( EE_ACTIONS::showNetlist,           ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( ACTIONS::zoomUndo,                  ENABLE( haveZoomUndo ) );
+    mgr->SetConditions( ACTIONS::zoomRedo,                  ENABLE( haveZoomRedo ) );
+    mgr->SetConditions( SCH_ACTIONS::toggleGrid,            CHECK( showGridCondition ) );
+    mgr->SetConditions( SCH_ACTIONS::toggleLegend,          CHECK( showLegendCondition ) );
+    mgr->SetConditions( SCH_ACTIONS::toggleDottedSecondary, CHECK( showDottedCondition ) );
+    mgr->SetConditions( SCH_ACTIONS::toggleDarkModePlots,   CHECK( darkModePlotCondition ) );
 
+    mgr->SetConditions( SCH_ACTIONS::newAnalysisTab,        ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    mgr->SetConditions( SCH_ACTIONS::simAnalysisProperties, ENABLE( haveSim ) );
+    mgr->SetConditions( SCH_ACTIONS::runSimulation,         ENABLE( !simRunning ) );
+    mgr->SetConditions( SCH_ACTIONS::stopSimulation,        ENABLE( simRunning ) );
+    mgr->SetConditions( SCH_ACTIONS::simProbe,              ENABLE( simFinished ) );
+    mgr->SetConditions( SCH_ACTIONS::simTune,               ENABLE( simFinished ) );
+    mgr->SetConditions( SCH_ACTIONS::showNetlist,           ENABLE( SELECTION_CONDITIONS::ShowAlways ) );
+    // clang-format on
 #undef CHECK
 #undef ENABLE
 }
