@@ -32,10 +32,11 @@
 #include <preview_items/snap_indicator.h>
 #include <preview_items/construction_geom.h>
 #include <tool/construction_manager.h>
-#include <tool/tool_manager.h>
 #include <tool/selection.h>
 #include <origin_viewitem.h>
 
+class TOOL_MANAGER; // Forward declaration to avoid hard dependency in tests
+
 class EDA_ITEM;
 
 enum GRID_HELPER_GRIDS : int
@@ -52,7 +53,9 @@ enum GRID_HELPER_GRIDS : int
 
 class GRID_HELPER
 {
+    friend void TEST_CLEAR_ANCHORS( GRID_HELPER& helper );
 public:
+    GRID_HELPER();
     GRID_HELPER( TOOL_MANAGER* aToolMgr, int aConstructionLayer );
     virtual ~GRID_HELPER();
 
@@ -60,23 +63,31 @@ public:
     VECTOR2D GetVisibleGrid() const;
     VECTOR2I GetOrigin() const;
 
+    // Manual setters used when no TOOL_MANAGER/View is available (e.g. in tests)
+    void SetGridSize( const VECTOR2D& aGrid ) { m_manualGrid = aGrid; }
+    void SetVisibleGridSize( const VECTOR2D& aGrid ) { m_manualVisibleGrid = aGrid; }
+    void SetOrigin( const VECTOR2I& aOrigin ) { m_manualOrigin = aOrigin; }
+    void SetGridSnapping( bool aEnable ) { m_manualGridSnapping = aEnable; }
+
     void SetAuxAxes( bool aEnable, const VECTOR2I& aOrigin = VECTOR2I( 0, 0 ) );
 
     virtual VECTOR2I Align( const VECTOR2I& aPoint, GRID_HELPER_GRIDS aGrid ) const
     {
-        return Align( aPoint, GetGridSize( aGrid ) );
+        return Align( aPoint, GetGridSize( aGrid ), GetOrigin() );
     }
 
     virtual VECTOR2I AlignGrid( const VECTOR2I& aPoint, GRID_HELPER_GRIDS aGrid ) const
     {
-        return AlignGrid( aPoint, GetGridSize( aGrid ) );
+        return AlignGrid( aPoint, GetGridSize( aGrid ), GetOrigin() );
     }
 
     virtual VECTOR2I Align( const VECTOR2I& aPoint ) const;
-    virtual VECTOR2I Align( const VECTOR2I& aPoint, const VECTOR2D& aGrid ) const;
+    virtual VECTOR2I Align( const VECTOR2I& aPoint, const VECTOR2D& aGrid,
+                            const VECTOR2D& aOffset ) const;
 
     VECTOR2I AlignGrid( const VECTOR2I& aPoint ) const;
-    VECTOR2I AlignGrid( const VECTOR2I& aPoint, const VECTOR2D& aGrid ) const;
+    VECTOR2I AlignGrid( const VECTOR2I& aPoint, const VECTOR2D& aGrid,
+                        const VECTOR2D& aOffset ) const;
 
     /**
      * Gets the coarsest grid that applies to a selecion of items.
@@ -113,6 +124,13 @@ public:
     bool GetUseGrid() const { return m_enableGrid; }
 
     void SetSnapLine( bool aSnap ) { m_enableSnapLine = aSnap; }
+    void SetSnapLineDirections( const std::vector<VECTOR2I>& aDirections );
+    void SetSnapLineOrigin( const VECTOR2I& aOrigin );
+    void ClearSnapLine();
+    std::optional<VECTOR2I> SnapToConstructionLines( const VECTOR2I& aPoint,
+                                                     const VECTOR2I& aNearestGrid,
+                                                     const VECTOR2D& aGrid,
+                                                     double aSnapRange ) const;
 
     void SetMask( int aMask ) { m_maskTypes = aMask; }
     void SetMaskFlag( int aFlag ) { m_maskTypes |= aFlag; }
@@ -197,7 +215,8 @@ protected:
      */
     bool canUseGrid() const;
 
-    VECTOR2I computeNearest( const VECTOR2I& aPoint, const VECTOR2I& aGrid ) const;
+    VECTOR2I computeNearest( const VECTOR2I& aPoint, const VECTOR2I& aGrid,
+                             const VECTOR2I& aOffset ) const;
 
 protected:
     void showConstructionGeometry( bool aShow );
@@ -230,6 +249,12 @@ protected:
     KIGFX::SNAP_INDICATOR   m_viewSnapPoint;
     KIGFX::ORIGIN_VIEWITEM  m_viewAxis;
 
+    // Manual grid parameters used when no TOOL_MANAGER is provided
+    VECTOR2D                m_manualGrid;
+    VECTOR2D                m_manualVisibleGrid;
+    VECTOR2I                m_manualOrigin;
+    bool                    m_manualGridSnapping;
+
 private:
     /// Show construction geometry (if any) on the canvas.
     KIGFX::CONSTRUCTION_GEOM m_constructionGeomPreview;
