@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2017 Jean_Pierre Charras <jp.charras at wanadoo.fr>
  * Copyright (C) 2015 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
- * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <pcb_track.h>
 #include <collectors.h>
 #include <reporter.h>
+#include <richio.h>
 
 #include <gendrill_file_writer_base.h>
 
@@ -146,8 +147,12 @@ void GENDRILL_WRITER_BASE::buildHolesList( DRILL_LAYER_PAIR aLayerPair,
                 new_hole.m_Hole_Diameter  = std::min( pad->GetDrillSize().x, pad->GetDrillSize().y );
                 new_hole.m_Hole_Size.x    = new_hole.m_Hole_Size.y = new_hole.m_Hole_Diameter;
 
-                if( pad->GetDrillShape() != PAD_DRILL_SHAPE_CIRCLE )
+                // Convert oblong holes that are actually circular into drill hits
+                if( pad->GetDrillShape() != PAD_DRILL_SHAPE::CIRCLE &&
+                        pad->GetDrillSizeX() != pad->GetDrillSizeY() )
+                {
                     new_hole.m_Hole_Shape = 1; // oval flag set
+                }
 
                 new_hole.m_Hole_Size         = pad->GetDrillSize();
                 new_hole.m_Hole_Pos          = pad->GetPosition();  // hole position
@@ -247,7 +252,11 @@ const std::string GENDRILL_WRITER_BASE::layerName( PCB_LAYER_ID aLayer ) const
     case B_Cu:
         return "back";
     default:
-        return StrPrintf( "in%d", aLayer );
+    {
+        // aLayer use even values, and the first internal layer (In1) is B_Cu + 2.
+        int ly_id = ( aLayer - B_Cu ) / 2;
+        return StrPrintf( "in%d", ly_id );
+    }
     }
 }
 
@@ -320,7 +329,7 @@ bool GENDRILL_WRITER_BASE::CreateMapFilesSet( const wxString& aPlotDirectory,
         // is created (do not create any PTH drill file can be seen as not working drill generator).
         if( getHolesCount() > 0 || doing_npth || pair == DRILL_LAYER_PAIR( F_Cu, B_Cu ) )
         {
-            fn = getDrillFileName( pair, doing_npth, m_merge_PTH_NPTH );
+            fn = GENDRILL_WRITER_BASE::getDrillFileName( pair, doing_npth, m_merge_PTH_NPTH );
             fn.SetPath( aPlotDirectory );
 
             fn.SetExt( wxEmptyString ); // Will be added by GenDrillMap
@@ -381,12 +390,18 @@ const wxString GENDRILL_WRITER_BASE::BuildFileFunctionAttributeString(
     // In Gerber files, layers num are 1 to copper layer count instead of F_Cu to B_Cu
     // (0 to copper layer count-1)
     // Note also for a n copper layers board, gerber layers num are 1 ... n
-    layer1 += 1;
+    //
+    // Copper layers use even values, so the layer id in file is
+    // (Copper layer id) /2 + 1 if layer is not B_Cu
+    if( layer1 == F_Cu )
+        layer1 = 1;
+    else
+        layer1 = ( ( layer1 - B_Cu ) / 2 ) + 1;
 
     if( layer2 == B_Cu )
         layer2 = m_pcb->GetCopperLayerCount();
     else
-        layer2 += 1;
+        layer2 = ( ( layer2 - B_Cu ) / 2) + 1;
 
     text << layer1 << wxT( "," ) << layer2;
 
@@ -434,7 +449,7 @@ const wxString GENDRILL_WRITER_BASE::BuildFileFunctionAttributeString(
     else if( hasDrill )
         text << wxT( ",Drill" );
     else if( hasOblong )
-        text << wxT( ",Route" );
+        text << wxT( ",Rout" );
 
     // else: empty file.
 
