@@ -134,6 +134,8 @@ public:
     FOOTPRINT& operator=( const FOOTPRINT& aOther );
     FOOTPRINT& operator=( FOOTPRINT&& aOther );
 
+    void CopyFrom( const BOARD_ITEM* aOther ) override;
+
     void Serialize( google::protobuf::Any &aContainer ) const override;
     bool Deserialize( const google::protobuf::Any &aContainer ) override;
 
@@ -142,6 +144,9 @@ public:
         return aItem && aItem->Type() == PCB_FOOTPRINT_T;
     }
 
+    /// Resets the caches for this footprint, for example if it was modified via the API
+    void InvalidateGeometryCaches();
+
     LSET GetPrivateLayers() const { return m_privateLayers; }
     void SetPrivateLayers( LSET aLayers ) { m_privateLayers = aLayers; }
 
@@ -497,6 +502,9 @@ public:
      */
     void CheckNetTiePadGroups( const std::function<void( const wxString& )>& aErrorHandler );
 
+    void CheckClippedSilk( const std::function<void( BOARD_ITEM* aItemA,
+                                                     BOARD_ITEM* aItemB,
+                                                     const VECTOR2I& aPt )>& aErrorHandler );
     /**
      * Cache the pads that are allowed to connect to each other in the footprint.
      */
@@ -534,14 +542,10 @@ public:
      *          Due to diff between layers and holes, these pads must be skipped to be sure
      *          there is no copper left on the board (for instance when creating Gerber Files or
      *          3D shapes).  Defaults to false.
-     * @param aSkipPlatedPads is used on 3D-Viewer to extract plated and non-plated pads.
-     * @param aSkipNonPlatedPads is used on 3D-Viewer to extract plated and plated pads.
      */
     void TransformPadsToPolySet( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
                                  int aMaxError, ERROR_LOC aErrorLoc,
-                                 bool aSkipNPTHPadsWihNoCopper = false,
-                                 bool aSkipPlatedPads = false,
-                                 bool aSkipNonPlatedPads = false ) const;
+                                 bool aSkipNPTHPadsWihNoCopper = false ) const;
 
     /**
      * Generate shapes of graphic items (outlines) on layer \a aLayer as polygons and adds these
