@@ -1,13 +1,16 @@
-import datetime
 import email
 import gzip
 import platform
 import re
 import sys
+from dataclasses import dataclass
+from datetime import datetime, UTC
+
+import msgpack
 
 from .conditions import is_android
 
-VERSION_RE = re.compile(r"(\d+\.\d+\.\d+)(?:[-\.]?)(.*)")
+VERSION_RE = re.compile(r"(\d+\.\d+\.\d+)[-.]?(.*)")
 
 
 def matches(actual, expected):
@@ -35,6 +38,18 @@ def assert_session(envelope, extra_assertion=None):
         assert_matches(session, extra_assertion)
 
 
+def assert_user_feedback(envelope):
+    user_feedback = None
+    for item in envelope:
+        if item.headers.get("type") == "user_report" and item.payload.json is not None:
+            user_feedback = item.payload.json
+
+    assert user_feedback is not None
+    assert user_feedback["name"] == "some-name"
+    assert user_feedback["email"] == "some-email"
+    assert user_feedback["comments"] == "some-comment"
+
+
 def assert_meta(
     envelope,
     release="test-example-release",
@@ -51,13 +66,16 @@ def assert_meta(
         "user": {"id": 42, "username": "some_name"},
         "transaction": transaction,
         "tags": {"expected-tag": "some value"},
-        "extra": {"extra stuff": "some value", "…unicode key…": "őá…–🤮🚀¿ 한글 테스트"},
+        "extra": {
+            "extra stuff": "some value",
+            "…unicode key…": "őá…–🤮🚀¿ 한글 테스트",
+        },
     }
     expected_sdk = {
         "name": "sentry.native",
-        "version": "0.6.0",
+        "version": "0.7.2",
         "packages": [
-            {"name": "github:getsentry/sentry-native", "version": "0.6.0"},
+            {"name": "github:getsentry/sentry-native", "version": "0.7.2"},
         ],
     }
     if is_android:
@@ -95,7 +113,7 @@ def assert_meta(
             )
             assert event["contexts"]["os"]["build"] is not None
 
-    if sdk_override != None:
+    if sdk_override is not None:
         expected_sdk["name"] = sdk_override
 
     assert_matches(event, expected)
@@ -139,6 +157,12 @@ def assert_breadcrumb(envelope):
         "message": "debug crumb",
         "category": "example!",
         "level": "debug",
+        "data": {
+            "url": "https://example.com/api/1.0/users",
+            "method": "GET",
+            "status_code": 200,
+            "reason": "OK",
+        },
     }
     assert any(matches(b, expected) for b in event["breadcrumbs"])
 
@@ -161,21 +185,29 @@ def assert_minidump(envelope):
     assert minidump.payload.bytes.startswith(b"MDMP")
 
 
-def assert_timestamp(ts, now=datetime.datetime.utcnow()):
+def assert_timestamp(ts, now=datetime.now(UTC)):
     assert ts[:11] == now.isoformat()[:11]
 
 
-def assert_event(envelope):
+def assert_event(envelope, message="Hello World!"):
     event = envelope.get_event()
     expected = {
         "level": "info",
         "logger": "my-logger",
-        "message": {"formatted": "Hello World!"},
+        "message": {"formatted": message},
     }
     assert_matches(event, expected)
     assert_timestamp(event["timestamp"])
 
 
+def assert_breakpad_crash(envelope):
+    event = envelope.get_event()
+    expected = {
+        "level": "fatal",
+    }
+    assert_matches(event, expected)
+
+
 def assert_exception(envelope):
     event = envelope.get_event()
     exception = {
@@ -186,7 +218,7 @@ def assert_exception(envelope):
     assert_timestamp(event["timestamp"])
 
 
-def assert_crash(envelope):
+def assert_inproc_crash(envelope):
     event = envelope.get_event()
     assert_matches(event, {"level": "fatal"})
     # depending on the unwinder, we currently don’t get any stack frames from
@@ -213,19 +245,76 @@ def assert_no_before_send(envelope):
     assert ("adapted_by", "before_send") not in event.items()
 
 
+@dataclass(frozen=True)
+class CrashpadAttachments:
+    event: dict
+    breadcrumb1: list
+    breadcrumb2: list
+
+
+def _unpack_breadcrumbs(payload):
+    unpacker = msgpack.Unpacker()
+    unpacker.feed(payload)
+    return [unpacked for unpacked in unpacker]
+
+
+def _load_crashpad_attachments(msg):
+    event = {}
+    breadcrumb1 = []
+    breadcrumb2 = []
+    for part in msg.walk():
+        if part.get_filename() is not None:
+            assert part.get("Content-Type") is None
+
+        match part.get_filename():
+            case "__sentry-event":
+                event = msgpack.unpackb(part.get_payload(decode=True))
+            case "__sentry-breadcrumb1":
+                breadcrumb1 = _unpack_breadcrumbs(part.get_payload(decode=True))
+            case "__sentry-breadcrumb2":
+                breadcrumb2 = _unpack_breadcrumbs(part.get_payload(decode=True))
+
+    return CrashpadAttachments(event, breadcrumb1, breadcrumb2)
+
+
+def is_valid_timestamp(timestamp):
+    try:
+        datetime.fromisoformat(timestamp)
+        return True
+    except ValueError:
+        return False
+
+
+def _validate_breadcrumb_seq(seq, breadcrumb_func):
+    for i in seq:
+        breadcrumb = breadcrumb_func(i)
+        assert breadcrumb["message"] == str(i)
+        assert is_valid_timestamp(breadcrumb["timestamp"])
+
+
 def assert_crashpad_upload(req):
     multipart = gzip.decompress(req.get_data())
     msg = email.message_from_bytes(bytes(str(req.headers), encoding="utf8") + multipart)
-    files = [part.get_filename() for part in msg.walk()]
+    attachments = _load_crashpad_attachments(msg)
 
-    # TODO:
-    # Actually assert that we get a correct event/breadcrumbs payload
-    assert "__sentry-breadcrumb1" in files
-    assert "__sentry-breadcrumb2" in files
-    assert "__sentry-event" in files
+    if len(attachments.breadcrumb1) > 3:
+        _validate_breadcrumb_seq(range(97), lambda i: attachments.breadcrumb1[3 + i])
+        _validate_breadcrumb_seq(
+            range(97, 101), lambda i: attachments.breadcrumb2[i - 97]
+        )
+
+    assert attachments.event["level"] == "fatal"
 
     assert any(
         b'name="upload_file_minidump"' in part.as_bytes()
         and b"\n\nMDMP" in part.as_bytes()
         for part in msg.walk()
     )
+
+
+def assert_gzip_file_header(output):
+    assert output[:3] == b"\x1f\x8b\x08"
+
+
+def assert_gzip_content_encoding(req):
+    assert req.content_encoding == "gzip"
